"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const imageInputFileToBase64_1 = __importDefault(require("./media/imageInputFileToBase64"));
const imageInputToBase64_1 = __importDefault(require("./media/imageInputToBase64"));
const inputFileToBase64_1 = __importDefault(require("./media/inputFileToBase64"));
const getAccessToken_1 = __importDefault(require("./auth/google/getAccessToken"));
const getAccessToken_2 = __importDefault(require("./auth/github/getAccessToken"));
const logout_1 = __importDefault(require("./auth/logout"));
const fetch_1 = __importDefault(require("./fetch"));
const fetch_2 = __importDefault(require("./fetch"));
const serialize_query_1 = __importDefault(require("../package-shared/utils/serialize-query"));
const serialize_cookies_1 = __importDefault(require("../package-shared/utils/serialize-cookies"));
const ejson_1 = __importDefault(require("../package-shared/utils/ejson"));
const numberfy_1 = __importDefault(require("../package-shared/utils/numberfy"));
const slugify_1 = __importDefault(require("../package-shared/utils/slugify"));
const post_login_1 = __importDefault(require("./auth/post-login"));
const deserialize_query_1 = __importDefault(require("../package-shared/utils/deserialize-query"));
const debug_log_1 = __importDefault(require("../package-shared/utils/logging/debug-log"));
const media = {
    imageInputToBase64: imageInputToBase64_1.default,
    imageInputFileToBase64: imageInputFileToBase64_1.default,
    inputFileToBase64: inputFileToBase64_1.default,
};
/**
 * User Auth Object
 */
const auth = {
    google: {
        getAccessToken: getAccessToken_1.default,
    },
    github: {
        getAccessToken: getAccessToken_2.default,
    },
    logout: logout_1.default,
    postLogin: post_login_1.default,
};
const utils = {
    deserializeQuery: deserialize_query_1.default,
    serializeQuery: serialize_query_1.default,
    serializeCookies: serialize_cookies_1.default,
    EJSON: ejson_1.default,
    numberfy: numberfy_1.default,
    slugify: slugify_1.default,
    debugLog: debug_log_1.default,
};
/**
 * Fetch
 */
const fetch = {
    fetchApi: fetch_1.default,
    clientFetch: fetch_2.default,
};
/**
 * Main Export
 */
const datasquirelClient = { media, auth, fetch, utils };
exports.default = datasquirelClient;
