"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = apiGet;
const lodash_1 = __importDefault(require("lodash"));
const serverError_1 = __importDefault(require("../../backend/serverError"));
const runQuery_1 = __importDefault(require("../../backend/db/runQuery"));
const grab_query_and_values_1 = __importDefault(require("../../../utils/grab-query-and-values"));
/**
 * # Get Function FOr API
 */
function apiGet(_a) {
    return __awaiter(this, arguments, void 0, function* ({ query, dbFullName, queryValues, tableName, dbSchema, debug, dbContext, forceLocal, }) {
        var _b, _c;
        const queryAndValues = (0, grab_query_and_values_1.default)({
            query,
            values: queryValues,
        });
        if (typeof query == "string" && query.match(/^alter|^delete|^create/i)) {
            return { success: false, msg: "Wrong Input." };
        }
        let results;
        try {
            let { result, error } = yield (0, runQuery_1.default)({
                dbFullName: dbFullName,
                query: queryAndValues.query,
                queryValuesArray: queryAndValues.values,
                readOnly: true,
                dbSchema,
                tableName,
                dbContext,
                debug,
                forceLocal,
            });
            if (debug && global.DSQL_USE_LOCAL) {
                console.log("apiGet:result", result);
                console.log("apiGet:error", error);
            }
            let tableSchema;
            if (dbSchema) {
                const targetTable = (_b = dbSchema.tables) === null || _b === void 0 ? void 0 : _b.find((table) => table.tableName === tableName);
                if (targetTable) {
                    const clonedTargetTable = lodash_1.default.cloneDeep(targetTable);
                    delete clonedTargetTable.childTable;
                    delete clonedTargetTable.childTableDbFullName;
                    delete clonedTargetTable.childTableName;
                    delete clonedTargetTable.childrenTables;
                    delete clonedTargetTable.updateData;
                    delete clonedTargetTable.tableNameOld;
                    delete clonedTargetTable.indexes;
                    tableSchema = clonedTargetTable;
                }
            }
            if (error)
                throw error;
            if (result.error)
                throw new Error(result.error);
            results = result;
            const resObject = {
                success: true,
                payload: results,
                schema: tableName && tableSchema ? tableSchema : undefined,
            };
            return resObject;
        }
        catch (error) {
            (0, serverError_1.default)({
                component: "/api/query/get/lines-85-94",
                message: error.message,
            });
            (_c = global.ERROR_CALLBACK) === null || _c === void 0 ? void 0 : _c.call(global, `API Get Error`, error);
            if (debug && global.DSQL_USE_LOCAL) {
                console.log("apiGet:error", error.message);
                console.log("queryAndValues", queryAndValues);
            }
            return {
                success: false,
                payload: null,
                error: error.message,
            };
        }
    });
}
