"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
require("dotenv").config({ path: "./../.env" });
const generate_password_1 = __importDefault(require("generate-password"));
const noDatabaseDbHandler_1 = __importDefault(require("./utils/noDatabaseDbHandler"));
const dbHandler_1 = __importDefault(require("./utils/dbHandler"));
const encrypt_1 = __importDefault(require("../functions/dsql/encrypt"));
/**
 * Create database from Schema Function
 * ==============================================================================
 * @param {object} params - Single object params
 * @param {number|string|null} params.userId - User ID or null
 */
function resetSQLCredentials() {
    return __awaiter(this, void 0, void 0, function* () {
        var _a;
        const users = (yield (0, dbHandler_1.default)({
            query: `SELECT * FROM users`,
        }));
        if (!users) {
            process.exit();
        }
        for (let i = 0; i < users.length; i++) {
            const user = users[i];
            if (!user)
                continue;
            const defaultMariadbUserHost = process.env.DSQL_DB_HOST || "127.0.0.1";
            try {
                const username = `dsql_user_${user.id}`;
                const password = generate_password_1.default.generate({
                    length: 16,
                    numbers: true,
                    symbols: true,
                    uppercase: true,
                    exclude: "*#.'`\"",
                });
                const encryptedPassword = (0, encrypt_1.default)({ data: password });
                yield (0, noDatabaseDbHandler_1.default)(`DROP USER IF EXISTS '${username}'@'%'`);
                yield (0, noDatabaseDbHandler_1.default)(`DROP USER IF EXISTS '${username}'@'${defaultMariadbUserHost}'`);
                yield (0, noDatabaseDbHandler_1.default)(`CREATE USER IF NOT EXISTS '${username}'@'${defaultMariadbUserHost}' IDENTIFIED BY '${password}'`);
                yield (0, noDatabaseDbHandler_1.default)(`GRANT ALL PRIVILEGES ON \`datasquirel_user_${user.id}_%\`.* TO '${username}'@'${defaultMariadbUserHost}'`);
                yield (0, noDatabaseDbHandler_1.default)(`FLUSH PRIVILEGES`);
                const updateUser = yield (0, dbHandler_1.default)({
                    query: `UPDATE users SET mariadb_user = ?, mariadb_host = ?, mariadb_pass = ? WHERE id = ?`,
                    values: [
                        username,
                        defaultMariadbUserHost,
                        encryptedPassword,
                        user.id,
                    ],
                });
                console.log(`User ${user.id}: ${user.first_name} ${user.last_name} SQL credentials successfully added.`);
            }
            catch (error) {
                (_a = global.ERROR_CALLBACK) === null || _a === void 0 ? void 0 : _a.call(global, `Error Resetting SQL credentials`, error);
                console.log(`Error in adding SQL user =>`, error.message);
            }
        }
        process.exit();
    });
}
resetSQLCredentials();
