import { DATASQUIREL_LoggedInUser, UserType } from "../../../types";
import path from "path";

type Param = {
    user?: DATASQUIREL_LoggedInUser | UserType;
    userId?: string | number | null;
    appDir?: string;
};
export default function grabDirNames(param?: Param) {
    const appDir = param?.appDir || process.env.DSQL_APP_DIR;
    const STATIC_ROOT = process.env.DSQL_STATIC_SERVER_DIR || "/static";

    const finalUserId = param?.user?.id || param?.userId;

    const publicImagesDir = path.join(STATIC_ROOT, `images`);

    if (!appDir)
        throw new Error("Please provide the `DSQL_APP_DIR` env variable.");

    const schemasDir =
        process.env.DSQL_DB_SCHEMA_DIR ||
        path.join(appDir, "jsonData", "dbSchemas");
    const tempDirName = ".tmp";

    if (!schemasDir)
        throw new Error(
            "Please provide the `DSQL_DB_SCHEMA_DIR` env variable."
        );

    const pakageSharedDir = path.join(appDir, `package-shared`);

    const mainDbTypeDefFile = path.join(pakageSharedDir, `types/dsql.ts`);
    const mainShemaJSONFilePath = path.join(schemasDir, `main.json`);
    const defaultTableFieldsJSONFilePath = path.join(
        pakageSharedDir,
        `data/defaultFields.json`
    );

    const usersSchemaDir = path.join(schemasDir, `users`);
    const targetUserSchemaDir = finalUserId
        ? path.join(usersSchemaDir, `user-${finalUserId}`)
        : undefined;
    const userTempSQLFilePath = targetUserSchemaDir
        ? path.join(targetUserSchemaDir, `tmp.sql`)
        : undefined;

    const userDirPath = finalUserId
        ? path.join(usersSchemaDir, `user-${finalUserId}`)
        : undefined;
    const userSchemaMainJSONFilePath = userDirPath
        ? path.join(userDirPath, `main.json`)
        : undefined;
    const userPrivateMediaDir = userDirPath
        ? path.join(userDirPath, `media`)
        : undefined;
    const userPrivateExportsDir = userDirPath
        ? path.join(userDirPath, `export`)
        : undefined;
    const userPrivateSQLExportsDir = userPrivateExportsDir
        ? path.join(userPrivateExportsDir, `sql`)
        : undefined;
    const userPrivateTempSQLExportsDir = userPrivateSQLExportsDir
        ? path.join(userPrivateSQLExportsDir, tempDirName)
        : undefined;
    const userPrivateTempJSONSchemaFilePath = userPrivateTempSQLExportsDir
        ? path.join(userPrivateTempSQLExportsDir, `schema.json`)
        : undefined;
    const userPrivateDbExportZipFileName = `db-export.zip`;
    const userPrivateDbExportZipFilePath = userPrivateSQLExportsDir
        ? path.join(userPrivateSQLExportsDir, userPrivateDbExportZipFileName)
        : undefined;

    const userPublicMediaDir = finalUserId
        ? path.join(publicImagesDir, `user-images/user-${finalUserId}`)
        : undefined;

    const userPrivateDbImportZipFileName = `db-export.zip`;
    const userPrivateDbImportZipFilePath = userPrivateSQLExportsDir
        ? path.join(userPrivateSQLExportsDir, userPrivateDbImportZipFileName)
        : undefined;

    const dbNginxLoadBalancerConfigFile = path.join(
        appDir,
        "docker/services/mariadb/load-balancer/config/template/nginx.conf"
    );

    let dockerComposeFile = path.join(appDir, "docker-compose.yml");
    let dockerComposeFileAlt = path.join(appDir, "docker-compose.yaml");
    const testDockerComposeFile = path.join(appDir, "test.docker-compose.yml");
    const testDockerComposeFileAlt = path.join(
        appDir,
        "test.docker-compose.yaml"
    );
    const extraDockerComposeFile = path.join(
        appDir,
        "extra.docker-compose.yml"
    );
    const extraDockerComposeFileAlt = path.join(
        appDir,
        "extra.docker-compose.yaml"
    );

    const siteSetupFile = path.join(appDir, "site-setup.json");

    const envFile = path.join(appDir, ".env");
    const testEnvFile = path.join(appDir, "test.env");

    return {
        appDir,
        schemasDir,
        userDirPath,
        mainShemaJSONFilePath,
        mainDbTypeDefFile,
        tempDirName,
        defaultTableFieldsJSONFilePath,
        usersSchemaDir,
        targetUserSchemaDir,
        userSchemaMainJSONFilePath,
        userPrivateMediaDir,
        userPrivateExportsDir,
        userPrivateSQLExportsDir,
        userPrivateTempSQLExportsDir,
        userPrivateTempJSONSchemaFilePath,
        userPrivateDbExportZipFileName,
        userPrivateDbExportZipFilePath,
        userPrivateDbImportZipFileName,
        userPrivateDbImportZipFilePath,
        dbNginxLoadBalancerConfigFile,
        dockerComposeFile,
        dockerComposeFileAlt,
        testDockerComposeFile,
        testDockerComposeFileAlt,
        extraDockerComposeFile,
        extraDockerComposeFileAlt,
        siteSetupFile,
        envFile,
        testEnvFile,
        userPublicMediaDir,
        userTempSQLFilePath,
    };
}
